@@ -1,10 +1,12 @@
-ivar <- function(x, ns=attr(x,"ns"), drop.levels=1){
+ivar <- function(x, ns=attr(x,"ns"), drop.levels=attr(x,"drop.levels")){
 #
 #   Rep a column vector into a matrix of correct size
 #
 
 if( is.null(ns) ) stop( paste("Number of occasions for", deparse(substitute(x)), "not specified"))
 
+if( is.null(drop.levels) ) drop.levels <- 1
+
 if(is.factor(x)){
     x <- as.factor(as.character(x))  # drop unused levels
     xm <- model.matrix( ~-1+x, data=x )   # no intercept, drop a level in F.cr.model.matrix if appropriate
